#ifndef FLYICON_H
#define FLYICON_H

#include <QIcon>

#include <fly/flyui_export.h>

class FLYUI_EXPORT FlyIcon
{
public:
    enum IconContext {
        AnyContext,
        Actions,
        Applications,
        Categories,
        Devices,
        Emblems,
        Emotes,
        International,
        MimeTypes,
        Places,
        Status
    };

    static QIcon fromTheme(const QString& name, IconContext context = AnyContext, const QIcon& fallback = QIcon());
    static QIcon fromTheme(const QString& name, IconContext context, const QStringList& overlays, const QIcon& fallback = QIcon());

    static QIcon forMimeType(const QString& mimeType, const QIcon& fallback = QIcon());
    static QIcon forMimeType(const QString& mimeType, const QStringList& overlays, const QIcon& fallback = QIcon());

    static QIcon forUrl(const QString& url, const QIcon& fallback = QIcon());
    static QIcon forUrl(const QString& url, const QStringList& overlays, const QIcon& fallback = QIcon());

    static QIcon withEmblems(const QIcon& icon, const QStringList& overlays);

    typedef int (*OverlaySizeMappingFunction)(int);
    static QIcon withOverlay(const QIcon& icon, Qt::Corner overlayCorner, const QIcon& overlayIcon,
                             OverlaySizeMappingFunction getOverlaySize = 0);

    static QString contextToString(IconContext context);
    static IconContext stringToContext(const QString& contextString);

    static bool hasThemeIcon(const QString& name, IconContext context = AnyContext);
    static bool hasMimeTypeIcon(const QString& mimeType);
    static bool hasUrlIcon(const QString& url);

private:
    FlyIcon();
    ~FlyIcon();
};

#endif // FLYICON_H
