//
// C++ Interface: shortcutsmodel
//
// Description:
//
//
// Author: Alexander Volkov <support@rusbitech.ru>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef SHORTCUTSMODEL_H
#define SHORTCUTSMODEL_H

#include <QAbstractItemModel>
#include <QStringList>

#include <fly/flyui_export.h>

class QShortcut;
class QAction;
class QSettings;

class IShortcutObject;

class FLYUI_EXPORT ShortcutsModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    ShortcutsModel(QSettings *settings, const QString settingsGroup = QString(), QObject *parent = 0);
    ~ShortcutsModel();

    void registerShortcut(const QString &id, QShortcut *shortcut,
                          const QString &description, const QString &category = QString());
    void registerAction(const QString &id, QAction *action, const QString &category = QString());

    bool isShortcutItem(const QModelIndex &index) const;
    void setKeySequence(const QModelIndex &index, const QKeySequence &ks);
    QKeySequence keySequence(const QModelIndex &index) const;
    QKeySequence defaultKeySequence(const QModelIndex &index) const;

    const IShortcutObject *hasKeySequence(const QKeySequence &seq) const;

    QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const Q_DECL_OVERRIDE;
    QModelIndex parent(const QModelIndex &index) const Q_DECL_OVERRIDE;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const Q_DECL_OVERRIDE;
    bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole) Q_DECL_OVERRIDE;
    int rowCount(const QModelIndex &parent = QModelIndex()) const Q_DECL_OVERRIDE;
    int columnCount(const QModelIndex &parent = QModelIndex()) const Q_DECL_OVERRIDE;
    Qt::ItemFlags flags(const QModelIndex &index) const Q_DECL_OVERRIDE;
    QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const Q_DECL_OVERRIDE;

public slots:
    void restoreDefaults();
    void resetShortcuts();
    void commitChanges();

    void saveKeys(QSettings *settings, const QString settingsGroup = QString());

private slots:
    void deleteShortcutObject();

private:
    void loadKeys(QSettings *settings, const QString settingsGroup = QString());
    void registerShortcutObject(IShortcutObject *o);

private:
    QMap<QString, QKeySequence> idToKeyMap;
    QStringList categories;
    typedef QList<IShortcutObject *> ShortcutObjectPtrList;
    QList< QMap<QString, ShortcutObjectPtrList> > shortcutsByCategories;
};

#endif
